#!/usr/bin/python3

import pycurl
from io import BytesIO
from utils import notify,debug,error

# args should be a dictionary of arguments
# return page bytes, response code
def download (platform, url, args, verbosity, return_http_code=False, follow_location=True):
	page_bytes = BytesIO()
	c = pycurl.Curl()

	c.setopt(c.URL, url)
	c.setopt(c.WRITEDATA, page_bytes)
	c.setopt(c.FOLLOWLOCATION, follow_location)

	# TODO: handle possible arguments
	if not args is None:
		if "user_agent" in args:
			c.setopt(pycurl.USERAGENT, args["user_agent"])
		if "header" in args:
			c.setopt(pycurl.HTTPHEADER, args["header"])
		if "proxy" in args and args["proxy"]:
			proxy_protocol = args["proxy_protocol"]
			if proxy_protocol == "HTTP":
				c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_HTTP)
			elif proxy_protocol == "SOCKS4":
				c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS4)
			elif proxy_protocol == "SOCKS4A":
				c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS4A)
			elif proxy_protocol == "SOCKS5":
				c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS5)
				notify ("Note, socks5 leaks DNS queries. Use socks5h to proxy DNS as well.", verbosity, platform)
			elif proxy_protocol == "SOCKS5H":
				c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS5_HOSTNAME)
			c.setopt(pycurl.PROXY, args["proxy_hostname"])
			c.setopt(pycurl.PROXYPORT, args["proxy_port"])
	notify ("Downloading " + url + "...", verbosity, platform)
	try:
		c.perform()
	except pycurl.error as e:
		error (str(e), verbosity, platform)
		return None
	response_code = int(c.getinfo(c.RESPONSE_CODE))
	c.close()
	debug (url + " downloaded!", verbosity, platform)
	if response_code in range(400,599):
		error ("Server returned " + str(response_code), verbosity, platform)
	else:
		debug ("Server returned " + str(response_code), verbosity, platform)
	if return_http_code:
		return page_bytes.getvalue().decode('utf8'),response_code
	else:
		return page_bytes.getvalue().decode('utf8')
